class PsenvParameterStoreError(Exception):
    pass


class PsenvConfigNotFoundError(Exception):
    pass


class PsenvConfigError(Exception):
    pass


class PsenvInternalError(Exception):
    pass


class PsenvInvalidAccountError(Exception):
    pass


class PsenvCLIError(Exception):
    pass
